# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010 AvanzOsc S.L. (http://avanzosc.com) All Rights Reserved.
#                       Ana Juaristi <anajuaristi@avanzosc.com>
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields,osv,orm
import xmlrpclib

class products_options(osv.osv):
    _name = "products.options"
    _description = "Generic Product Options"

    _columns = {
        'web_id': fields.many2one( 'esale.oscom.web', 'Website' ),
        'name' : fields.char('Option', size=64),
        'products_options_id' : fields.integer('Osc Option ID'),
        'esale_oscom_id' : fields.integer('Osc Language'),

    }
products_options()

class product_variant_dimension_type(osv.osv):
    _inherit = 'product.variant.dimension.type'
    _columns = {
        'products_options_id' : fields.many2one('products.options', 'OSC Option'),
    }
    def onchange_products_options_id(self, cr, uid, ids, products_options_id, name, context={}):
        if onchange_products_options_id:
            opt=self.pool.get('products.options').browse(cr,uid,[products_options_id])[0]           
            if not name:
                v['name'] = opt.name
            return {'value': v}
        return {}

product_variant_dimension_type()


class products_options_values(osv.osv):
    _name = "products.options.values"
    _description = "Generic Product Options Values"

    _columns = {
        'web_id': fields.many2one( 'esale.oscom.web', 'Website' ),
        'name' : fields.char('Option Values', size=64),
        'products_options_values_id' : fields.integer('Osc Option Values ID'),
        'esale_oscom_id' : fields.integer('Osc Language'),
    }

products_options_values()


class product_variant_dimension_value(osv.osv):
    _inherit = "product.variant.dimension.value"
    _columns = {
        'products_options_values_id' : fields.many2one('products.options.values', 'Osc Option Value'),
    }
    def onchange_products_options_values_id(self, cr, uid, ids, products_options_values_id, name, context={}):
        if onchange_products_options_values_id:
            optv=self.pool.get('products.options_values').browse(cr,uid,[products_options_values_id])[0]           
            if not name:
                v['name'] = optv.name
            return {'value': v}
        return {}

product_variant_dimension_value()

class products_attributes(osv.osv):
    _name = "products.attributes"
    _description = "Product Atributes"
    
     
    _columns = {
        'web_id': fields.many2one( 'esale.oscom.web', 'Website' ),
        'esale_oscom_id' : fields.integer('Osc Product ID'),
        'esale_options_id' : fields.integer('Osc Option ID'),
        'esale_options_values_id' : fields.integer('Osc Option Values ID'),
        'esale_options_values_price' : fields.float('Options price', (15,4)),
        'esale_price_prefix' : fields.char('Price prefix', size=1),
        'esale_prod_id' : fields.many2one('esale.oscom.product', 'Osc Product Id'),
        'products_options_id' : fields.many2one('products.options', 'Option'),
        'products_options_values_id' : fields.many2one('products.options.values', 'Option values'),
    }

    def onchange_esale_oscom_id(self, cr, uid, ids, web_id, esale_oscom_id, esale_prod_id, context={}):
        if onchange_esale_options_id:

            esale_product_ids = esale_product_obj.search(cr, uid, [('web_id','=', web_id), ('esale_oscom_id','=',esale_oscom_id)])
            if esale_product_ids:
                esale_product = esale_product_obj.browse(cr, uid, esale_product_ids[0])
                v['esale_prod_id'] = esale_product.product_id.id
            return {'value': v}
        return {}

    def onchange_esale_options_id(self, cr, uid, ids, web_id, esale_options_id, products_options_id, context={}):
        if onchange_esale_options_id:

            esale_options_ids = esale_options_obj.search(cr, uid, [('web_id','=', web_id), ('products_options_id','=',esale_options_id)])
            if esale_options_ids :
                esale_options = esale_options_obj.browse(cr, uid, esale_options_ids[0])
                v['products_options_id'] = esale_options.products_options_id.id
            return {'value': v}
        return {}

    def onchange_esale_options_values_id(self, cr, uid, ids, web_id, esale_options_values_id, products_options_values_ids, context={}):
        if onchange_esale_optionsproducts_options_values_id_values_id:

            esale_options_values_ids = esale_options_values_obj.search(cr, uid, [('web_id','=', web_id), ('products_options_values_id','=',esale_options_values_id)])
            if esale_options_values_ids :
                esale_options_values = esale_options_values_obj.browse(cr, uid, esale_options_values_ids[0])
                v['products_options_values_id'] = esale_options_values.products_options_id.id
            return {'value': v}
        return {}


products_attributes()

class products_esale_oscom_variants(osv.osv):
    _inherit = "product.product"

    _columns = {
        'attributes_ids' : fields.one2many('products.attributes', 'esale_prod_id', 'Languages'),

    }
products_esale_oscom_variants()


class product_template(osv.osv):
    _inherit = "product.template"

product_template()

class esale_oscom_web(osv.osv):
    _inherit = "esale.oscom.web"

    _columns = {
        'products_options_ids' : fields.one2many('products.options', 'products_options_id', 'Options'),
        'products_options_values_ids' : fields.one2many('products.options.values', 'products_options_values_id', 'Options Values'),
        'attributes_ids' : fields.one2many('products.attributes', 'esale_oscom_id', 'Products Attributes'),

    }
    def get_options(self, cr, uid, ids, *args):
        esale_options_obj = self.pool.get('products.options')

        for website in self.browse(cr, uid, ids):
            server = xmlrpclib.ServerProxy("%s/openerp-synchro.php" % website.url)
            esale_options = server.get_options()
            for esale_option in esale_options:
                value={
                    'web_id' : website.id,
                    'name' : esale_option[1],
                    'products_options_id' : esale_option[0],
                    'esale_oscom_id': esale_option[2]
                 }
                existing = esale_options_obj.search(cr, uid, [('web_id','=',website.id), ('products_options_id', '=', esale_option[0]), ('esale_oscom_id', '=', esale_option[2])])
                if len(existing) > 0:
                    esale_options_obj.write(cr, uid, existing, value)
                    #print "Encontrada opcion: ", esale_option[1] 
                else:
                    esale_options_obj.create(cr, uid, value)
                    #print "NO Encontrada opcion: ", esale_option[1]  
        return True



    def get_options_values(self, cr, uid, ids, *args):
        esale_options_values_obj = self.pool.get('products.options.values')
        esale_lang_obj = self.pool.get('esale.oscom.lang')

        for website in self.browse(cr, uid, ids):
            server = xmlrpclib.ServerProxy("%s/openerp-synchro.php" % website.url)
            esale_options_values = server.get_options_values()
            for esale_options_value in esale_options_values:
                value={
                    'web_id' : website.id,
                    'name' : esale_options_value[1],
                    'products_options_values_id' : esale_options_value[0],
                    'esale_oscom_id' : esale_options_value[2],
                }
                existing = esale_options_values_obj.search(cr, uid, [('web_id','=',website.id), ('products_options_values_id', '=', esale_options_value[0]), ('esale_oscom_id', '=', esale_options_value[2])])
                if len(existing) > 0:
                    esale_options_values_obj.write(cr, uid, existing, value)
                    #print "Encontrada opción: ", esale_options_value[1]

                else:
                    esale_options_values_obj.create(cr, uid, value)
                    #print "NO Encontrada opción: ", esale_options_value[1]

        return True

    def get_attributes(self, cr, uid, ids, *args):
        esale_attributes_obj = self.pool.get('products.attributes')
        esale_products_obj = self.pool.get('esale.oscom.product')
        esale_options_obj = self.pool.get('products.options')
        esale_options_values_obj = self.pool.get('products.options.values')

        for website in self.browse(cr, uid, ids):
            server = xmlrpclib.ServerProxy("%s/openerp-synchro.php" % website.url)
            esale_attributes = server.get_attributes()
            for esale_attribute in esale_attributes:
                print "esale_attribute", esale_attribute

                prod_ids = esale_products_obj.search(cr, uid, [('web_id','=',website.id), ('esale_oscom_id', '=', esale_attribute[0])])
                print "Producto: ", prod_ids
                if prod_ids:
                    prod = esale_products_obj.browse(cr, uid, prod_ids[0])

                opt_ids = esale_options_obj.search(cr, uid, [('web_id','=',website.id), ('products_options_id', '=', esale_attribute[1])])
                print "Options: ", opt_ids
                if opt_ids:
                    opt = esale_options_obj.browse(cr, uid, opt_ids[0])

                optv_ids = esale_options_values_obj.search(cr, uid, [('web_id','=',website.id), ('products_options_values_id', '=', esale_attribute[2])])
                print "Options: ", optv_ids
                if optv_ids:
                    optv = esale_options_values_obj.browse(cr, uid, optv_ids[0])

                    value={
                        'web_id': website.id,
                        'esale_oscom_id' : esale_attribute[0],
                        'esale_options_id' : esale_attribute[1],
                    }    
                    prod_ids = esale_products_obj.search(cr, uid, [('web_id','=',website.id), ('esale_oscom_id', '=', esale_attribute[0])])
                print "Producto: ", prod_ids
                if prod_ids:
                    prod = esale_products_obj.browse(cr, uid, prod_ids[0])
                    value={                
                        'esale_options_values_id' : esale_attribute[2],
                        'esale_options_values_price' : esale_attribute[3],
                        'esale_price_prefix' : esale_attribute[4],
                        'esale_prod_id' : prod.id,
                        'products_options_id' : opt.id,
                        'products_options_values_id' : optv.id,
                        'esale_oscom_id' : esale_attribute[0],
                    }
                existing = esale_attributes_obj.search(cr, uid, [('web_id','=',website.id), ('esale_oscom_id', '=', esale_attribute[0]),('esale_options_id', '=', esale_attribute[1]),('esale_options_values_id', '=', esale_attribute[2])])
                if len(existing) > 0:
                    esale_attributes_obj.write(cr, uid, existing, value)
                else:
                    esale_attributes_obj.create(cr, uid, value)
        return True

    def product_variant_exists(self, cr, uid, ids, osc_product_variants):
        esale_attributes_obj = self.pool.get('products.attributes')

        existing = esale_attributes_obj.search(cr, uid, [('web_id','=',website.id), ('esale_oscom_id', '=', osc_product_variants['products_id']),('esale_options_id', '=', osc_product_variants['products_options_id']),('esale_options_values_id', '=',  osc_product_variants['products_options_values_id'])])
        return (len(existing) > 0)        

    def set_extra_price(self, cr, uid, product):
        dimension_extra = 0.0
        for dim in product.dimension_value_ids:
            dimension_extra += dim.price_extra         

        return dimension_extra

    def variant_import_create(self, cr, uid, website, prod_id, product, esale_product):
        template_dimensions_obj = self.pool.get('product.variant.dimension.type')
        template_values_obj = self.pool.get('product.variant.dimension.value')
        template_obj = self.pool.get('product.template')
        product_obj = self.pool.get('product.product')
        esale_products_obj = self.pool.get('esale.oscom.product')
        esale_options_obj = self.pool.get('products.options')
        esale_options_values_obj = self.pool.get('products.options.values')
        template_list = []
        oerp_langs = {}
        for lang in website.language_ids:
            if lang.language_id and lang.language_id.translatable:
                oerp_langs[lang.esale_oscom_id] = lang.language_id.id
        print "oerp_langs", oerp_langs
        print "==========Variantes =========" 
        #Duplicates product for each osc attribute, associating with same OSC object
        print "==========Variantes ========="
        print product['variants']

        oerp_product = product_obj.browse(cr, uid, prod_id)  
        template_id = oerp_product.product_tmpl_id.id
        template_obj = template_obj.browse(cr, uid, template_id)
        

        for variant in product['variants']:
            #print "==========Inicio tratamiento variantes ========="
            
            print "==========web_id =========", website.id
            #print "==========name =========", variant['products_options_name']
            print "==========esale_oscom_id =========", oerp_langs[variant['opt_language_id']]         
                
            opt_ids = esale_options_obj.search(cr, uid, [('web_id','=',website.id), ('name', '=', variant['products_options_name']), ('esale_oscom_id', '=', variant['opt_language_id'])])
 
            print "In Options: ", opt_ids
            if opt_ids:
                opt = esale_options_obj.browse(cr, uid, opt_ids[0])
                #print "Option Found: ", opt.name
 
            tmp_dim_ids = template_dimensions_obj.search(cr, uid, [('product_tmpl_id','=', template_id ), ('name', '=', variant['products_options_name'])])
            value={'name' : variant['products_options_name'],
               'products_options_id' : opt.id,
               'product_tmpl_id' : template_id,
               }
            #print "value dimension", value
            if len(tmp_dim_ids)>0:
                dim_id = tmp_dim_ids[0]
                tmp_dim = template_dimensions_obj.browse(cr, uid, tmp_dim_ids[0])
                template_dimensions_obj.write(cr, uid, tmp_dim.id, value)
                dim_id= tmp_dim.id
                #print "Encontrada dimension para template: ", tmp_dim.name
                #print "Encontrada dimension para template de otro modo: ", tmp_dim_ids[0] 

            else:
                dim_id = template_dimensions_obj.create(cr, uid, value)
                #print "NO Encontrada Dimension, creada: ", dim_id 

            optv_ids = esale_options_values_obj.search(cr, uid, [('web_id','=',website.id), ('name', '=', variant['opv.products_options_values_name'])])
            print "Options VALUES: ", optv_ids
            if optv_ids:
                optv = esale_options_values_obj.browse(cr, uid, optv_ids[0])
                #print "Options VALUES: ", optv.name
            tmp_val_ids = template_values_obj.search(cr, uid, [('product_tmpl_id','=',template_id), ('dimension_id', '=', dim_id), ('name', '=', variant['opv.products_options_values_name'])])
            value={'name' : variant['opv.products_options_values_name'],
               'dimension_id' : dim_id,
               'products_options_values_id' : optv.id,
               'product_tmpl_id' : template_id,
               'price_extra' : variant['options_values_price']
               }
            #print "value dimension value", value
            #print "tmp_val_ids", tmp_val_ids

            if len(tmp_val_ids)>0:
                print "Encontrado valor de dimension para template: ", tmp_val_ids[0]
                tmp_val = template_values_obj.browse(cr, uid, tmp_val_ids[0])
                val_id = tmp_val.id
                template_values_obj.write(cr,  uid, val_id, value )
                #print "Encontrado valor de dimension para template: ", tmp_val.name
            else:
                val_id = template_values_obj.create(cr, uid, value)
                #print "NO Encontrado valor de dimension para template: ", val_id
        print "Inicio creación variantes: " 
        
       
        print "generando variantes template", template_id
        self.pool.get('product.template').button_generate_variants(cr, uid, [template_id])
        print "Variantes generadas"
        template_products_objs_ids = product_obj.search(cr, uid, [('product_tmpl_id','=',template_id)])

        print "Asignando a variantes generadas:" , esale_product.esale_oscom_id       
        template_products_objs = product_obj.browse(cr, uid, template_products_objs_ids)
        print "oerp_product manufacturer", oerp_product.manufacturer_id.id
        
        for product in template_products_objs :
            value = {'price_extra' : self.set_extra_price(cr, uid, product),
                'price_margin': oerp_product.price_margin, 
                'default_code':oerp_product.default_code ,
                'weight': oerp_product.weight,
                'product_picture': oerp_product.product_picture,
                'in_out_stock': oerp_product.in_out_stock,
                'date_available': oerp_product.date_available,
                'spe_price_status': oerp_product.spe_price_status or False,
                'spe_price': oerp_product.spe_price or False,
                'exp_date': oerp_product.exp_date or False,
                'oscom_url': oerp_product.oscom_url ,
                'manufacturer_id': oerp_product.manufacturer_id.id,
                }
            product_obj.write(cr, uid, [product.id], value)
            #print "Tratando producto", product

        print "Asignando producto oscommerce a productos oerp: ", template_products_objs_ids
        esale_products_objs_ids = esale_products_obj.search(cr, uid, [('product_id', 'in', template_products_objs_ids)]) 
        value_esale = {
            'esale_oscom_id':esale_product.esale_oscom_id,
        }

        esale_products_obj.write(cr, uid, esale_products_objs_ids, value_esale)

esale_oscom_web()
